@@ -1,8 +1,10 @@
+from abnosql.mocks.mock_azure_kms import mock_azure_kms
 from abnosql.mocks.mock_cosmos import mock_cosmos
 from abnosql.mocks.mock_dynamodbx import mock_dynamodbx
 
 
 __all__ = [  # type: ignore
+    mock_azure_kms,
     mock_dynamodbx,
     mock_cosmos
 ]
