@@ -11,17 +11,17 @@ import pluggy  # type: ignore
 
 import abnosql.exceptions as ex
 from abnosql.plugin import PM
-from abnosql.table import add_audit
-from abnosql.table import check_exists
 from abnosql.table import check_exists_enabled
+from abnosql.table import delete_item_post
+from abnosql.table import delete_item_pre
+from abnosql.table import get_item_post
+from abnosql.table import get_item_pre
 from abnosql.table import get_key_attrs
 from abnosql.table import get_sql_params
-from abnosql.table import kms_decrypt_item
-from abnosql.table import kms_encrypt_item
 from abnosql.table import kms_process_query_items
+from abnosql.table import put_item_post
+from abnosql.table import put_item_pre
 from abnosql.table import TableBase
-from abnosql.table import validate_item
-from abnosql.table import validate_key_attrs
 from abnosql.table import validate_query_attrs
 
 hookimpl = pluggy.HookimplMarker('abnosql.table')
@@ -78,16 +78,16 @@ def dynamodb_ex_handler(raise_not_found: t.Optional[bool] = True):
             except ClientError as e:
                 code = e.response['Error']['Code']
                 if raise_not_found and code in ['ResourceNotFoundException']:
-                    raise ex.NotFoundException(e) from None
+                    raise ex.NotFoundException() from None
                 elif code == 'UnrecognizedClientException':
-                    raise ex.ConfigException(e) from None
-                raise ex.ValidationException(e) from None
+                    raise ex.ConfigException(detail=e) from None
+                raise ex.ValidationException(detail=e) from None
             except NoCredentialsError as e:
-                raise ex.ConfigException(e) from None
+                raise ex.ConfigException(detail=e) from None
             except ex.NoSQLException:
                 raise
             except Exception as e:
-                raise ex.PluginException(e)
+                raise ex.PluginException(detail=e)
         return wrapper
     return decorator
 
@@ -151,11 +151,13 @@ def get_dynamodb_kwargs(
 
 class Table(TableBase):
 
+    @dynamodb_ex_handler()
     def __init__(
         self, pm: PM, name: str, config: t.Optional[dict] = None
     ) -> None:
         self.pm = pm
         self.name = name
+        self.database = 'dynamodb'
         self.set_config(config)
         self.session = self.config.get(
             'session', boto3.session.Session(
@@ -177,19 +179,15 @@ class Table(TableBase):
 
     @dynamodb_ex_handler()
     def get_item(self, **kwargs) -> t.Optional[t.Dict]:
+        audit_key, _ = get_item_pre(self, dict(**kwargs))
+
         response = deserialize(self.table.get_item(
             TableName=self.name,
             Key=get_key(**kwargs)
         ), self.config.get('deserializer'))
-        _check_exists = dict(**kwargs).pop('abnosql_check_exists', None)
         item = response.get('Item')
-        _item = self.pm.hook.get_item_post(table=self.name, item=item)
-        if _item:
-            item = _item
-        item = kms_decrypt_item(self.config, item)
-        if _check_exists is not False:
-            check_exists(self, 'get', item)
-        return item
+
+        return get_item_post(self, dict(**kwargs), item, audit_key)
 
     @dynamodb_ex_handler()
     def put_item(
@@ -198,19 +196,11 @@ class Table(TableBase):
         update: t.Optional[bool] = False,
         audit_user: t.Optional[str] = None
     ) -> t.Dict:
-        operation = 'update' if update else 'create'
-        validate_item(self.config, operation, item)
-        item = check_exists(self, operation, item)
-        if audit_user:
-            item = add_audit(item, update or False, audit_user)
-        _item = self.pm.hook.put_item_pre(table=self.name, item=item)
-        if _item:
-            item = _item[0]
-        item = kms_encrypt_item(self.config, item)
+        item, _ = put_item_pre(self, item, update, audit_user)
+        item = json.loads(json.dumps(item), parse_float=Decimal)
 
         # do update
         if update is True:
-            validate_key_attrs(self.key_attrs, item)
             kwargs = {
                 'Key': {k: item.pop(k) for k in self.key_attrs},
                 'ReturnValues': 'ALL_NEW'
@@ -234,8 +224,7 @@ class Table(TableBase):
         else:
             self.table.put_item(Item=item)
 
-        self.pm.hook.put_item_post(table=self.name, item=item)
-        return item
+        return put_item_post(self, item, update, audit_user)
 
     @dynamodb_ex_handler()
     def put_items(
@@ -251,10 +240,11 @@ class Table(TableBase):
 
     @dynamodb_ex_handler()
     def delete_item(self, **kwargs):
-        check_exists(self, 'delete', dict(kwargs))
-        key = get_key(**kwargs)
-        self.table.delete_item(Key=key)
-        self.pm.hook.delete_item_post(table=self.name, key=key)
+        key = delete_item_pre(self, dict(kwargs))
+
+        self.table.delete_item(Key=get_key(**kwargs))
+
+        delete_item_post(self, key)
 
     @dynamodb_ex_handler()
     def query(
