@@ -37,6 +37,18 @@ class TableSpecs(plugin.PluginSpec):
         """
         pass
 
+    @hookspec(firstresult=True)
+    def get_item_pre(self, table: str, key: t.Dict):  # type: ignore[empty-body] # noqa E501
+        """Hook invoked before get_item()
+
+        Args:
+
+            table: table name
+            key: dictionary of key passed to get_item()
+
+        """
+        pass
+
     @hookspec(firstresult=True)
     def get_item_post(self, table: str, item: t.Dict) -> t.Dict:  # type: ignore[empty-body] # noqa E501
         """Hook invoked after get_item()
@@ -137,8 +149,7 @@ class TableBase(metaclass=ABCMeta):
     def put_item(
         self,
         item: t.Dict,
-        update: t.Optional[bool] = False,
-        audit_user: t.Optional[str] = None
+        update: t.Optional[bool] = False
     ) -> t.Dict:
         """Puts table/collection item
 
@@ -285,6 +296,70 @@ def quote_str(string):
     ) + "'"
 
 
+def get_item_pre(tb, kwargs):
+    key = validate_key_attrs(tb.key_attrs, dict(**kwargs), False)
+    tb.pm.hook.get_item_pre(table=tb.name, key=key)
+    _check_exists = kwargs.pop('abnosql_check_exists', None)
+    return key, _check_exists
+
+
+def get_item_post(tb, kwargs, item, audit_key):
+    _check_exists = kwargs.pop('abnosql_check_exists', None)
+    _item = tb.pm.hook.get_item_post(table=tb.name, item=item)
+    if _item:
+        item = _item
+    item = kms_decrypt_item(tb.config, item)
+    if _check_exists is not False:
+        check_exists(tb, 'get', item)
+    if item is not None:
+        audit_callback(tb, 'get', audit_key)
+    return item
+
+
+def put_item_pre(tb, item, update, audit_user):
+    operation = 'update' if update else 'create'
+    key = validate_key_attrs(tb.key_attrs, item)
+    validate_item(tb.config, operation, item)
+    item = check_exists(tb, operation, item)
+
+    audit_user = audit_user or tb.config.get('audit_user')
+    if audit_user:
+        item = add_audit(item, update or False, audit_user)
+
+    # add change metadata if enabled
+    if hasattr(tb, 'change_meta') and tb.change_meta is True:
+        item = add_change_meta(
+            item, tb.name, 'MODIFY' if update is True else 'INSERT'
+        )
+
+    _item = tb.pm.hook.put_item_pre(table=tb.name, item=item)
+    if _item:
+        item = _item[0]
+    item = kms_encrypt_item(tb.config, item)
+    return item, key
+
+
+def put_item_post(tb, item, update, audit_user, abnosql_audit_callback=True):
+    key = validate_key_attrs(tb.key_attrs, item)
+    tb.pm.hook.put_item_post(table=tb.name, item=item)
+    if abnosql_audit_callback is not False:
+        audit_callback(
+            tb, 'update' if update else 'create', key, audit_user
+        )
+    return item
+
+
+def delete_item_pre(tb, kwargs):
+    key = validate_key_attrs(tb.key_attrs, dict(**kwargs), False)
+    check_exists(tb, 'delete', dict(kwargs))
+    return key
+
+
+def delete_item_post(tb, key):
+    tb.pm.hook.delete_item_post(table=tb.name, key=key)
+    audit_callback(tb, 'delete', key)
+
+
 def validate_query_attrs(key: t.Dict, filters: t.Dict):
     """Validate that the query and filter attributes are named correctly
 
@@ -344,10 +419,16 @@ def get_key_attrs(config: t.Optional[t.Dict] = None) -> list:
         key_attrs = _key_attrs
     if len(key_attrs) > 2:
         raise ValueError('must not be more than 2 key_attrs defined')
+    if len(key_attrs) == 0:
+        raise ValueError('key_attrs not defined')
     return key_attrs
 
 
-def validate_key_attrs(key_attrs: t.Iterable[str], item: t.Dict):
+def validate_key_attrs(
+    key_attrs: t.Iterable[str],
+    item: t.Dict,
+    is_put: t.Optional[bool] = True
+) -> t.Dict:
     """Validate key attributes in either ABNOSQL_KEY_ATTRS env var or config
 
     Args:
@@ -365,10 +446,12 @@ def validate_key_attrs(key_attrs: t.Iterable[str], item: t.Dict):
         raise ex.ValidationException(
             f'key_attrs missing from item: {missing}'
         )
-    if len(item) - len(key_attrs) <= 0:
+    if is_put and len(item) - len(key_attrs) <= 0:
         raise ex.ValidationException(
             'item contains no additional keys beyond key_attrs'
         )
+    key = {k: item[k] for k in key_attrs}
+    return key
 
 
 def validate_item(
@@ -377,6 +460,12 @@ def validate_item(
     schema = config.get(f'{operation}_schema', config.get('schema'))
     if schema is None:
         return
+    camel_case = os.environ.get('ABNOSQL_CAMELCASE', 'TRUE') == 'TRUE'
+    meta_attr = 'changeMetadata' if camel_case else 'change_metadata'
+    name_attr = 'eventName' if camel_case else 'event_name'
+    existing_event = item.get(meta_attr, {}).get(name_attr)
+    if existing_event == 'REMOVE':
+        return
     title = config.get(
         f'{operation}_schema_errmsg',
         config.get('schema_errmsg', 'invalid item')
@@ -562,6 +651,16 @@ def add_change_meta(item: t.Dict, event_source: str, event_name: str) -> t.Dict:
     return item
 
 
+def audit_callback(tb, operation, key, audit_user=None):
+    _callback = tb.config.get('audit_callback')
+    audit_user = audit_user or tb.config.get('audit_user')
+    if not callable(_callback) or not isinstance(audit_user, str):
+        return
+    dt_iso = datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%SZ')
+    key_str = ';'.join([f'{k}={v}' for k, v in sorted(key.items())])
+    _callback(tb.name, dt_iso, operation, key_str, audit_user)
+
+
 def check_exists_enabled(config):
     return config.get(
         'check_exists',
@@ -570,7 +669,11 @@ def check_exists_enabled(config):
 
 
 def check_exists(obj: TableBase, operation: str, item: dict):
-    if len(obj.key_attrs) == 0 or obj.check_exists is False:  # type: ignore
+    if (
+        len(obj.key_attrs) == 0   # type: ignore
+        or not hasattr(obj, 'check_exists')
+        or obj.check_exists is False
+    ):
         return item
     key = {
         k: item.get(k) for k in obj.key_attrs  # type: ignore
@@ -627,7 +730,8 @@ def table(
     if database is None:
         defaults = {
             'AWS_DEFAULT_REGION': 'dynamodb',
-            'FUNCTIONS_WORKER_RUNTIME': 'cosmos'
+            'FUNCTIONS_WORKER_RUNTIME': 'cosmos',
+            'K_SERVICE': 'firestore'
         }
         for envvar, _database in defaults.items():
             if os.environ.get(envvar) is not None:
@@ -654,7 +758,8 @@ def table(
     if isinstance(kcfg, dict):
         defaults = {
             'dynamodb': 'aws',
-            'cosmos': 'azure'
+            'cosmos': 'azure',
+            'firestore': 'gcp'
         }
         provider = kcfg.get('provider')
         if database is not None and provider is None:
